use std::{collections::HashMap, ops::Deref};

use mongod::ToAPI;
use rocket::{get, State};
use serde_json::json;

use crate::{
    check_auth,
    config::Config,
    json_store::JSONStore,
    location::Location,
    routes::{api_error, FallibleApiResponse, Token},
};

#[get("/location/<id>")]
pub async fn location_info(
    id: &str,
    locations: &State<JSONStore<Location>>,
    t: Token,
    c: &State<Config>,
) -> FallibleApiResponse {
    check_auth!(t, c);

    let loc = locations
        .get(id)
        .ok_or_else(|| api_error("No location with that ID"))?;
    Ok(loc.api().await)
}

/// Resolve locations children recursively
async fn location_api_resolve(
    id: &str,
    location: &Location,
    locations: &HashMap<String, Location>,
) -> serde_json::Value {
    let mut loc_api = location.api().await;
    let mut sub_l = HashMap::new();

    for sub_loc in locations {
        if let Some(parent) = &sub_loc.1.parent {
            if parent == id {
                sub_l.insert(
                    sub_loc.0.clone(),
                    Box::pin(location_api_resolve(&sub_loc.0, &sub_loc.1, locations)).await,
                );
            }
        }
    }

    loc_api
        .as_object_mut()
        .unwrap()
        .insert("sub".to_string(), serde_json::to_value(sub_l).unwrap());
    return loc_api;
}

#[get("/locations")]
pub async fn locations_list(
    locations: &State<JSONStore<Location>>,
    t: Token,
    c: &State<Config>,
) -> FallibleApiResponse {
    check_auth!(t, c);

    let mut lst = Vec::with_capacity(locations.len());

    for id in locations.deref().keys() {
        lst.push(id);
    }

    Ok(json!(lst))
}

#[get("/location_map")]
pub async fn locations_info(
    locations: &State<JSONStore<Location>>,
    t: Token,
    c: &State<Config>,
) -> FallibleApiResponse {
    check_auth!(t, c);

    let mut root_locations = HashMap::new();

    for loc in locations.inner().deref() {
        if loc.1.parent.is_none() {
            root_locations.insert(loc.0, loc.1);
        }
    }

    let mut location_api = HashMap::new();

    for loc in &mut root_locations {
        location_api.insert(
            loc.0,
            location_api_resolve(loc.0, loc.1, locations.inner()).await,
        );
    }

    Ok(serde_json::to_value(location_api).unwrap())
}
