mod demand;
mod error;
mod location;
mod supply;

pub use demand::*;
pub use error::*;
pub use location::*;
use mongod::Model;
use mongod::ToAPI;
pub use supply::*;

use rocket::get;
use rocket::State;
use serde_json::json;

use crate::check_auth;
use crate::config::Config;
use crate::db::ItemDB;
use crate::routes::Token;
use crate::transaction::Transaction;

use super::api_error;
use crate::routes::FallibleApiResponse;

/// Returns a JSON response with all items in the database.
#[get("/items")]
pub fn get_items_route(itemdb: &State<ItemDB>, t: Token, c: &State<Config>) -> FallibleApiResponse {
    check_auth!(t, c);

    let items = itemdb.items();
    Ok(json!({"items": items}))
}

/// Return an API Response for an `Item`
#[get("/item/<item_id>")]
pub fn item_route(
    item_id: &str,
    itemdb: &State<ItemDB>,
    t: Token,
    c: &State<Config>,
) -> FallibleApiResponse {
    check_auth!(t, c);

    let item = itemdb
        .get_item(item_id)
        .ok_or_else(item_does_not_exist_error)?;
    Ok(item.api_json())
}

/// Returns all variants of an Item
#[get("/item/<item_id>/variants")]
pub fn item_variants_page(
    item_id: &str,
    itemdb: &State<ItemDB>,
    t: Token,
    c: &State<Config>,
) -> FallibleApiResponse {
    check_auth!(t, c);

    let item = itemdb
        .get_item(item_id)
        .ok_or_else(item_does_not_exist_error)?;
    let variants = item.get_variants();

    Ok(json!({
        "item": item_id,
        "variants": variants
    }))
}

/// Returns an API Response for a `Transaction`
#[get("/transaction/<transaction>")]
pub async fn transaction_route(
    transaction: &str,
    t: Token,
    c: &State<Config>,
) -> FallibleApiResponse {
    check_auth!(t, c);

    let t = Transaction::get(transaction)
        .await
        .ok_or_else(|| api_error("No transaction with this UUID"))?;
    Ok(t.api().await)
}

/// Returns unique values for a field
#[get("/item/<item_id>/<variant_id>/unique?<field>")]
pub async fn unique_field_route(
    item_id: &str,
    variant_id: &str,
    field: &str,
    itemdb: &State<ItemDB>,
    t: Token,
    c: &State<Config>,
) -> FallibleApiResponse {
    check_auth!(t, c);

    let variant = itemdb
        .get_item(item_id)
        .ok_or_else(item_does_not_exist_error)?
        .variant(variant_id)
        .ok_or_else(variant_does_not_exist_error)?;

    match field {
        "origin" => Ok(json!(variant.get_unique_origins().await)),
        "destination" => Ok(json!(variant.get_unique_destinations().await)),
        _ => Err(api_error("Unknown field")),
    }
}
